some edits to NewFuzzer
[c11tester.git] / main.cc
diff --git a/main.cc b/main.cc
index cc3fcbf081316cf3958cf1f59915e4b3ca8d9e9a..590a41d27f33c07118592912fc73f582a4624050 100644 (file)
--- a/main.cc
+++ b/main.cc
 #include "snapshot-interface.h"
 #include "plugins.h"
 
-static void param_defaults(struct model_params *params)
+void param_defaults(struct model_params *params)
 {
        params->verbose = !!DBG_ENABLED();
        params->uninitvalue = 0;
        params->maxexecutions = 10;
        params->nofork = false;
+       params->threadsnocleanup = false;
 }
 
 static void print_usage(const char *program_name, struct model_params *params)
@@ -56,6 +57,9 @@ static void print_usage(const char *program_name, struct model_params *params)
                "                            Default: %u\n"
                "                            -o help for a list of options\n"
                "-n                          No fork\n"
+#ifdef TLS
+               "-d                          Don't allow threads to cleanup\n"
+#endif
                " --                         Program arguments follow.\n\n",
                program_name,
                params->verbose,
@@ -86,7 +90,7 @@ bool install_plugin(char * name) {
 
 static void parse_options(struct model_params *params, int argc, char **argv)
 {
-       const char *shortopts = "hnt:o:u:x:v::";
+       const char *shortopts = "hdnt:o:u:x:v::";
        const struct option longopts[] = {
                {"help", no_argument, NULL, 'h'},
                {"verbose", optional_argument, NULL, 'v'},
@@ -103,6 +107,9 @@ static void parse_options(struct model_params *params, int argc, char **argv)
                case 'h':
                        print_usage(argv[0], params);
                        break;
+               case 'd':
+                       params->threadsnocleanup = true;
+                       break;
                case 'n':
                        params->nofork = true;
                        break;
@@ -161,16 +168,6 @@ static void install_trace_analyses(ModelExecution *execution)
        }
 }
 
-/** The model_main function contains the main model checking loop. */
-static void model_main()
-{
-       snapshot_record(0);
-       model->run();
-       delete model;
-
-       DEBUG("Exiting\n");
-}
-
 /**
  * Main function.  Just initializes snapshotting library and the
  * snapshotting library calls the model_main function.
@@ -181,42 +178,37 @@ int main(int argc, char **argv)
        main_argv = argv;
 
        /*
* If this printf statement is removed, CDSChecker will fail on an
         * assert on some versions of glibc.  The first time printf is
         * called, it allocated internal buffers.  We can't easily snapshot
         * libc since we also use it.
         */
 
-       printf("CDSChecker\n"
-                                "Copyright (c) 2013 Regents of the University of California. All rights reserved.\n"
+       printf("C11Tester\n"
+                                "Copyright (c) 2013 and 2019 Regents of the University of California. All rights reserved.\n"
                                 "Distributed under the GPLv2\n"
-                                "Written by Brian Norris and Brian Demsky\n\n");
+                                "Written by Weiyu Luo, Brian Norris, and Brian Demsky\n\n");
 
-       /* Configure output redirection for the model-checker */
-       redirect_output();
 
-       //Initialize snapshotting library
-       if (!model_init)
+       //Initialize snapshotting library and model checker object
+       if (!model) {
                snapshot_system_init(10000, 1024, 1024, 40000);
+               model = new ModelChecker();
+               model->startChecker();
+       }
 
-       struct model_params params;
+       /* Configure output redirection for the model-checker */
+       redirect_output();
 
-       param_defaults(&params);
        register_plugins();
-       parse_options(&params, main_argc, main_argv);
 
-       //Initialize race detector
-       initRaceDetector();
+       //Parse command line options
+       model_params *params = model->getParams();
+       parse_options(params, main_argc, main_argv);
 
-       snapshot_stack_init();
 
-       if (!model_init)
-               model = new ModelChecker();
-       else
-               model = model_init;
-
-       model->setParams(params);
        install_trace_analyses(model->get_execution());
 
-       startExecution(&model_main);
+       model->startMainThread();
+       DEBUG("Exiting\n");
 }
@@ -181,42 +178,37 @@ int main(int argc, char **argv)
 
        /*
-        * If this printf statement is removed, CDSChecker will fail on an
+        * If this printf statement is removed, C11Tester will fail on an
         * assert on some versions of glibc.  The first time printf is
         * called, it allocated internal buffers.  We can't easily snapshot